/**
 * Java
 *
 * Copyright 2008-2013 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.elflw;


public interface ElfConstants {

	// --- Elf header	
	
	public static final int Sizeof32bitsHeader = 52;
	
	int EI_MAG0 		= 0;
    int EI_MAG1 		= 1;
    int EI_MAG2 		= 2;
    int EI_MAG3 		= 3;
    
    int SIGNATURE_LENGTH = 4;
    
    int EI_CLASS 		= 4;
    int EI_DATA 		= 5;
    int EI_VERSION 		= 6;
    int EI_OSABI 		= 7;
    int EI_ABIVERSION 	= 8;
    int EI_PAD 			= 9;
    int EI_NIDENT 		= 16;    
    
    int ELFMAG0 		= (byte)0x7F;
    int ELFMAG1 		= (byte)'E';
    int ELFMAG2 		= (byte)'L';
    int ELFMAG3 		= (byte)'F';
        
    int ELFCLASSNONE 	= 0;
    int ELFCLASS32 		= 1;
    int ELFCLASS64 		= 2;

    int ELFDATA2LSB 	= 1;
    int ELFDATA2MSB 	= 2;

    // --- e_type
	int ET_NONE			= 0;		//no type file
	int ET_REL 			= 1;		//relocatable file
	int ET_EXEC 		= 2;		//executable file
	int ET_DYN			= 3;		//shared object file
	int ET_CORE 		= 4;		//core file
	int ET_LOPROC 		= 0xFF00;	//processor specific
	int ET_HIPROC 		= 0xFFFF;	//processor specific
	int ET_LOOS 		= 0xFE00;	//os specific
	int ET_HIOS 		= 0xFEFF;	//os specific
   
	// --- e_machine
	int EM_NONE			= 0;
    
    // --- e_version
	int EV_NONE			= 0;
	int EV_CURRENT		= 1;

	// --- sections
	int Sizeof32bitsSectionHeader = 40;
	
	int SHN_UNDEF 		= 0;		//Undefined section reference
	int SHN_LORESERVE	= 0xFF00;	//Begin range of reserved indices
	int SHN_LOPROC 		= 0xFF00;	//Begin range of appl-specific
	int SHN_HIPROC 		= 0xFF1F;	//End range of appl-specific
	int SHN_ABS 		= 0xFFF1;	//Associated symbol is absolute
	int SHN_COMMON 		= 0xFFF2;	//Associated symbol is in common
	int SHN_XINDEX 		= 0xFFFF;
	int SHN_LOOS 		= 0xFF20;
	int SHN_HIOS 		= 0xFF3F;
	int SHN_HIRESERVE 	= 0xFFFF;	//End range of reserved indices
	
	// possible value for sh_type
	int SHT_NULL 			= 0;			// marks the section header as inactive, other values of the section header are undefined
	int SHT_PROGBITS 		= 1;			// The section holds information defined by the program, whose format and meaning are determined solely by the program.
	int SHT_SYMTAB 			= 2;			// the section holds a symbol table
	int SHT_STRTAB 			= 3;			// the section holds a string table
	int SHT_RELA			= 4;	    	// The section holds relocation entries with explicit addends, such as type
	int SHT_HASH			= 5;			// the section holds a symbol hash table
	int SHT_DYNAMIC 		= 6;			// the section holds information for dynamic linking
	int SHT_NOTE			= 7;			// the section holds information that marks the file in some way
	int SHT_NOBITS	 		= 8;			// A section of this type occupies no space in the file but otherwise resembles SHT_PROGBITS. Although this section contains no bytes, the sh_offset member contains the conceptual file offset.
	int SHT_REL		 		= 9;			// The section holds relocation entries without explicit addends, such as type Elf32_Rel for the 32-bit class of object files. An object file may have multiple relocation sections.
	int SHT_SHLIB			= 10;			// this section type is reserved but has unspecified semantics
	int SHT_DYNSYM 			= 11;			// the section holds a symbol table
	int SHT_INIT_ARRAY		= 14;			// Array of ptrs to init functions
	int SHT_FINI_ARRAY		= 15;			// Array of ptrs to finish functions
	int SHT_PREINIT_ARRAY 	= 16;			// Array of ptrs to pre-init funcsint SHT_GROUP	  		= 17;			// Section contains a section group
	int SHT_GROUP		 	= 17;			
	int SHT_SYMTAB_SHNDX 	= 18;			// Indicies for SHN_XINDEX entries
	int SHT_LOPROC 			= 0x70000000;	// Values in this inclusive range are reserved for processor-specific semantics
	int SHT_HIPROC	 		= 0x7FFFFFFF;	// idem
	int SHT_LOUSER 			= 0x80000000;	// This value specifies the lower bound of the range of indexes reserved for application programs.
	int SHT_HIUSER	 		= 0xFFFFFFFF;	// This value specifies the upper bound of the range of indexes reserved for
											// application programs. Section types between SHT_LOUSER and
											// SHT_HIUSER may be used by the application, without conflicting with
											// current or future system-defined section types.

	// possible values for sh_flags
	int SHF_WRITE				= 0x1;			// section contains data that should be writable during process execution
	int SHF_ALLOC				= 0x2;			// section occupies memory during process execution. Some control sections do not reside in the memory image of an object file; this attribute is off for those sections
	int SHF_EXECINSTR			= 0x4;			// section contains executable machine instructions
	int SHF_MERGE 				= 0x10;
	int SHF_STRINGS 			= 0x20;
	int SHF_INFO_LINK 			= 0x40;
	int SHF_LINK_ORDER 			= 0x80;
	int SHF_OS_NONCONFORMING  	= 0x100;
	int SHF_GROUP 				= 0x200;
	int SHF_TLS 				= 0x400;
	int SHF_MASKOS 				= 0x0FF00000;
	int SHF_ORDERED 			= 0x40000000;
	int SHF_EXCLUDE 			= 0x80000000;	
	int SHF_MASKPROC			= 0xf0000000;	//all bits included in this mask are reserved for processor-specific semantics

	// --- Symbol table entry
	// - binding
	public final static int STB_LOCAL 	= 0;
	public final static int STB_GLOBAL	= 1;
	public final static int STB_WEAK	= 2;
	public final static int STB_LOPROC	= 13;
	public final static int STB_HIPROC	= 15;
	
	// -type
	int STT_NOTYPE	= 0;
	int STT_OBJECT	= 1;
	int STT_FUNC	= 2;	
	int STT_SECTION	= 3;	
	int STT_FILE	= 4;	
	int STT_LOPROC	= 13;
	int STT_HIPROC	= 15;
	
	// - visibility
	int STV_DEFAULT		= 0;
	int STV_INTERNAL	= 1;
	int STV_HIDDEN		= 2;
	int STV_PROTECTED	= 3;

	// ---------------- Program Header (executable ELF) ----------------------------------
	// 32 bytes for each segment
	public static final int Sizeof32bitsProgramHeader = 32;
	// --- PT_XXX constants
	// The array element is unused; other members' values are undefined. This type lets the program header table have ignored entries.
	public final static int PT_NULL					= 0; 
	// The array element specifies a loadable segment, described by p_filesz and
	// p_memsz. The bytes from the file are mapped to the beginning of the memory
	// segment. If the segment's memory size (p_memsz) is larger than the file size
	// (p_filesz), the "extra'' bytes are defined to hold the value 0 and to follow the
	// segment's initialized area. The file size may not be larger than the memory size.
	// Loadable segment entries in the program header table appear in ascending order,
	// sorted on the p_vaddr member.
	public final static int PT_LOAD		= 1; 
	//	The array element specifies dynamic linking information.
	public final static int PT_DYNAMIC	= 2;						
	//	The array element specifies the location and size of a null-terminated path name to invoke as an interpreter.
	public final static int PT_INTERP	= 3;						
	//	The array element specifies the location and size of auxiliary information.
	public final static int PT_NOTE		= 4;						
	//	This segment type is reserved but has unspecified semantics.
	public final static int PT_SHLIB	= 5;				
	// The array element, if present, specifies the location and size of the program header
	// table itself, both in the file and in the memory image of the program. This segment
	// type may not occur more than once in a file. Moreover, it may occur only if the
	// program header table is part of the memory image of the program. If it is present,
	// it must precede any loadable segment entry.
	public final static int PT_PHDR		= 6;						
	// Thread local storage segment
	public final static int PT_TLS		= 7;				
	// Values in this inclusive range are reserved for processor-specific semantics.
	public final static int PT_LOPROC	= 0x70000000;		
	public final static int PT_HIPROC	= 0x7fffffff;			

	// --- PF_XXX constants
	public final static int PF_X			= 0x1;				// execute
	public final static int PF_W			= 0x2;				// write
	public final static int PF_R			= 0x4;				// read
	public final static int PF_MASKPROC		= 0xf0000000;	    // unspecified

	
	// --- Group section
	int GRP_COMDAT = 0x1;
	
	
	int SizeofSymbolTableEntry 		= 16;
	int SizeofRelocationEntry 		= 8;
	int SizeofRelocationEntryAddend = 12;
	
	// ELF spec : sections should be aligned on 2 bytes (at least)
	int DefaultSectionPadding = 2; 
}
