/*
 * Java
 *
 * Copyright 2008-2022 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.elflw.input.soar;

import com.is2t.elflw.input.AbstractElfLoader;
import com.is2t.elflw.input.soar.nodes.ElfGeneratorSoar;
import com.is2t.elflw.input.soar.nodes.RelocationSectionSoar;
import com.is2t.elflw.nodes.Section;
import com.is2t.elflw.support.SectionHeader;

import ej.annotation.Nullable;

public abstract class ElfLoaderSoar extends AbstractElfLoader implements ElfSoarMicroJvmConstants, ElfGeneratorSoar {

	public ElfLoaderSoar() {
		super();
	}

	@Override
	public int supportedMachine() {
		return EM_SOARMICROJVM;
	}

	@Override
	public String getTag() {
		return "SOAR";
	}

	@Override
	public void generateSoarRelocationSection(RelocationSectionSoar node) {
		SectionHeader sh = currentSectionHeader();
		node.targetSection = getExpectedProgSection(sh.link);
	}

	/**
	 * Gets the section index targetted by the given relocation section.
	 *
	 * @param sh
	 * @return the section index or <code>0</code> if the given section header is not a relocation section
	 */
	public int getRelocatedSection(SectionHeader sh) {
		int type = sh.type;
		int targetSectionIndex;
		if (type == SHT_REL || type == SHT_REL) {
			targetSectionIndex = sh.info;
			assert targetSectionIndex != 0;
		} else if (type == SECTION_TYPE_REL_SOAR) {
			targetSectionIndex = sh.link;
			assert targetSectionIndex != 0;
		} else {
			targetSectionIndex = 0;
		}
		return targetSectionIndex;
	}

	@Override
	@Nullable
	protected Section createCustomSection(SectionHeader sh, byte[] bytes) {
		if (sh.type == SECTION_TYPE_REL_SOAR) {
			return new RelocationSectionSoar(sh.addralign, bytes);
		} else {
			return super.createCustomSection(sh, bytes);
		}
	}
}
