/*
 * Java
 *
 * Copyright 2008-2022 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.elflw.nodes;

import com.is2t.elflw.ElfConstants;
import com.is2t.elflw.ElfGenerator;

import ej.annotation.Nullable;

/**
 * Abstract ELF unit (relocatable or executable) Currently 32bits oriented
 */
public abstract class ElfUnit extends ElfLinkerNode implements ElfConstants {

	public int targetId; // ELF target ID
	public boolean lsbAndNotMsb; // endianness (true littleEndian)
	public int flags; // header flags (interpretation is target specific)
	public final int abiVersion; // ABI version

	@Nullable
	public Section[] sections; // all sections from ELF index 1->NbSections-1 (first section is implicit) see section()

	/**
	 * Direct link to the first encountered {@link SymbolTableSection}.
	 */
	@Nullable
	public SymbolTableSection symbolTableSection;

	public ElfUnit(int targetId, boolean lsbAndNotMsb, int abiVersion, int flags) {
		this.targetId = targetId;
		this.lsbAndNotMsb = lsbAndNotMsb;
		this.abiVersion = abiVersion;
		this.flags = flags;
	}

	/**
	 * Return the section at the given index, <code>null</code> for index SHN_UNDEF (0) or reserved indexes
	 */
	@Nullable
	public Section section(int elfSectionIndex) {
		// Assume sections have been created
		if (elfSectionIndex == SHN_UNDEF) {
			return null;
		} else if ((elfSectionIndex & SHN_LORESERVE) == SHN_LORESERVE) {
			// reserved indices
			return null;
		} else {
			return getSections()[elfSectionIndex - 1];
		}
	}

	@Override
	public abstract void generateUsing(ElfGenerator gen);

	public static int elfSectionIndex(int zeroBasedIndex) {
		// sections are 1 based in the ELF file (0 is for special section)
		// call this method rather than inlining +1
		return zeroBasedIndex + 1;
	}

	/**
	 * Gets the loaded sections
	 *
	 * @return the loaded sections
	 */
	public Section[] getSections() {
		Section[] sections = this.sections;
		assert sections != null;
		return sections;
	}

}
