/*
 * Java
 *
 * Copyright 2008-2022 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.elflw.nodes;

import com.is2t.elflw.ElfGenerator;

import ej.annotation.Nullable;

public class RelocationEntry extends ElfLinkerNode {

	// The behavior is target dependent => moved into RelocationOperation objects
	// and the difference between Rel and Rela entries is the way to retrieve the addend
	public int offset;
	@Nullable
	public SymbolTableEntry symbolEntry; // null when this relocation does not need a symbol
	public int info; // Only the real info part : relocation code without symbolTable index == r_info&0xFF

	/**
	 * Has a mean only if its parent {@link RelocationSection#withAddend} is true.
	 */
	public int addend;

	public RelocationEntry(int offset, @Nullable SymbolTableEntry symbol, int info, int addend) {
		this.offset = offset;
		this.symbolEntry = symbol;
		this.info = info;
		this.addend = addend;
	}

	@Override
	public void generateUsing(ElfGenerator gen) {
		gen.generateRelocationEntry(this);
	}

	// Relocation entry r_info field : symbolTableIndex [bits 8->31] | info [bits 7->0]
	public static int ELF32_R_SYM(int i) {
		// return the symbolTable index
		return (i >>> 8);
	}

	public static int ELF32_R_TYPE(int i) {
		// return the target specific relation info
		return i & 0xFF;
	}

	public static int ELF32_R_INFO(int sym, int type) {
		return sym << 8 | type;
	}

}
