/*
 * Java
 *
 * Copyright 2015-2022 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.kf;

import java.util.ArrayList;

import com.is2t.elflw.allocator.SectionAllocator;

import ej.annotation.Nullable;

public class NativeSectionAllocator implements SectionAllocator {

	@Nullable
	private ArrayList<NativeMemoryBlock> allocatedBlocks;

	public NativeSectionAllocator() {
		this.allocatedBlocks = new ArrayList<NativeMemoryBlock>();
	}

	@Override
	@Nullable
	public NativeMemoryBlock allocate(int alignment, int size) {
		long memoryBlockDescriptor = DynamicLoaderNatives.allocate(alignment, size);
		if (memoryBlockDescriptor == 0) {
			return null;
		}
		NativeMemoryBlock b = new NativeMemoryBlock(memoryBlockDescriptor,
				DynamicLoaderNatives.getSectionStartAddress(memoryBlockDescriptor), size);
		ArrayList<NativeMemoryBlock> allocatedBlocks = this.allocatedBlocks;
		assert allocatedBlocks != null;
		allocatedBlocks.add(b);
		return b;
	}

	@Override
	public void reclaim() {
		ArrayList<NativeMemoryBlock> allocatedBlocks = this.allocatedBlocks;
		assert allocatedBlocks != null;
		for (NativeMemoryBlock m : allocatedBlocks) {
			DynamicLoaderNatives.free(m.memoryBlockDescriptor);
		}
		this.allocatedBlocks = null;
	}
}
