/*
 * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
 * Copyright 2019-2022 MicroEJ Corp. This file has been modified and/or created by MicroEJ Corp.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation. The Copyright holder(s) designates
 * this particular file as subject to the "Classpath" exception as provided
 * by the Copyright holder(s) in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact the Copyright holder(s) if you need additional information
 * or have any questions.
 * (if Oracle: 500 Oracle Parkway, Redwood Shores, CA 94065 USA or visit www.oracle.com)
 */

package com.is2t.support.net.ssl.util;

import java.io.IOException;
import java.net.SocketTimeoutException;
import java.security.KeyManagementException;

import javax.net.ssl.SSLException;
import javax.net.ssl.SSLHandshakeException;

import com.is2t.support.net.ssl.SSLErrorConstants;
import com.is2t.support.net.ssl.SSLErrorMessages;

import ej.error.Message;
import ej.sni.NativeIOException;

/**
 *
 */
public class NativeIOExceptionHandler {

	public static SSLException generateNativeSSLException(NativeIOException cause) {
		SSLException ex = new SSLException(generateNativeIOExceptionMessage(cause));
		ex.initCause(cause);
		return ex;
	}

	public static KeyManagementException generateNativeKeyManagementException(NativeIOException cause) {
		KeyManagementException ex = new KeyManagementException(generateNativeIOExceptionMessage(cause));
		ex.initCause(cause);
		return ex;
	}

	public static SSLHandshakeException generateNativeSSLHandshakeException(NativeIOException cause) {
		SSLHandshakeException ex = new SSLHandshakeException(generateNativeIOExceptionMessage(cause));
		ex.initCause(cause);
		return ex;
	}

	public static RuntimeException generateNativeRuntimeException(NativeIOException cause) {
		RuntimeException ex = new RuntimeException(generateNativeIOExceptionMessage(cause));
		ex.initCause(cause);
		return ex;
	}

	public static IOException generateNativeIOException(NativeIOException cause) {
		int errCode = cause.getErrorCode();
		String exceptionMsg = generateNativeIOExceptionMessage(cause);

		if (errCode == SSLErrorConstants.J_SOCKET_TIMEOUT) {
			SocketTimeoutException ex = new SocketTimeoutException(exceptionMsg);
			ex.initCause(cause);
			return ex;
		}

		IOException ex = new IOException(exceptionMsg);
		ex.initCause(cause);
		return ex;
	}

	private static String generateNativeIOExceptionMessage(NativeIOException cause) {
		int errCode = cause.getErrorCode();
		return Message.at(SSLErrorMessages.getInstance(), errCode);
	}
}
