/*
 * Java
 *
 * Copyright 2019-2022 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.kf;

import java.io.IOException;
import java.io.InputStream;

import ej.kf.AlreadyLoadedFeatureException;
import ej.kf.Feature;
import ej.kf.IncompatibleFeatureException;
import ej.kf.InvalidFormatException;

/**
 * Dynamic load of a {@link Feature}.
 * Data is a FS3 (which is a JAR).
 */
public class DynamicFeatureLoader implements IFeatureLoader {

	/** The JAR magic numbers. */
	private static final byte[] SIGNATURE = {0x50 , 0x4B, 0x03, 0x04};

	@Override
	public boolean canLoad(byte[] signature) {
		if (signature.length < SIGNATURE.length) {
			return false;
		}

		for (int i = 0; i < SIGNATURE.length; i++) {
			if (signature[i] != SIGNATURE[i]) {
				return false;
			}
		}

		return true;
	}

	@Override
	public boolean canUnload() {
		return true;
	}

	@Override
	public Feature load(byte[] signature, InputStream is)
			throws IOException, InvalidFormatException, IncompatibleFeatureException, AlreadyLoadedFeatureException {

		final int id = DynamicLoaderNatives.openDynamicFeatureTransfer();

		DynamicLoaderNatives.transferDynamicFeature(id, SIGNATURE, SIGNATURE.length);

		byte[] buffer = new byte[1024];
		int read = is.read(buffer);
		while (read != -1) {
			DynamicLoaderNatives.transferDynamicFeature(id, buffer, read);
			read = is.read(buffer);
		}

		long vmDescriptor = DynamicLoaderNatives.closeDynamicFeatureTransfer(id);
		return new Feature(this, vmDescriptor, Feature.FEATURE_CUSTOM_HANDLE_NONE);
	}

	@Override
	public boolean unload(Feature feature) {
		return DynamicLoaderNatives.unloadDynamicFeature(feature);
	}

	@Override
	public void loadInstalledFeatures() {
		// NOP
	}
}
